/*  Multiple precision integer arithmetic - see multip1.notes	*/
/*  Source:  miarith.c		Header:  miarith.h		*/
/*  Ernest Sibert -- February 1995				*/

/*  The notation  x[0..N] means that x consists of a status word x[0]	*/
/*  and N significant words x[1..N].  In consequence, x must have been	*/
/*  defined in C as  x[N+1]  (or larger).				*/
/*  Where dimensions are not mentioned, everything is [0..N].		*/
/*  In a few cases N is not relevant, since x[0] is all that matters.	*/


/*  Integer  d := x + y, may overflow	*/

extern void imadd(int d[], int x[], int y[], int N);


/*  Integer  d := -n, may overflow	*/

extern void imneg(int d[], int x[], int  N);


/*  Integer  d := x - y, may overflow	*/

extern void imsub(int d[], int x[], int y[], int N);


/*  d := x for multiple-precision	*/

extern void  mcopy(int d[], int x[], int N);


/*  d := 0 for multiple-precision	*/

extern void  mzero(int d[], int N);


/*  Integer  dd[0..M+N] := x[0..M]*y[0..N], for	*/
/*  x >= 0 and y >= 0, whence result is >= 0.	*/
/*  Neither x nor y should interfere with dd.	*/
/*  Overflow is impossible.			*/
/*  M < N is better than M > N if either is	*/
/*  likely to be small.				*/

extern void imult2x(int dd[], int x[], int y[], int M, int N);


/*  Integer  dd[0..2*N] := x[0..N]*y[0..N]			*/
/*  Multiplication by the most negative integer yelds overflow,	*/
/*  all other cases succeed without overflow.			*/

extern void imult2(int dd[], int x[], int y[], int M, int N);


/*  x[0..N] := s*x[0..N] + d  where  x >= 0 and  0 <= d < s < 2^Mhalf .	*/
/*  May overflow							*/

extern void immuls(int x[], int N, int s, int d);


/*  Shift, d := x*2^e, may generate overflow.	*/

extern void imshift(int d[], int x[], int e, int N);


/*  Report status of mp quantity as:		*/
/*       4   overflow				*/
/*	 1   positive				*/
/*	 0   zero				*/
/*	-1   negative, but not most negative	*/
/*	-2   most negative integer 		*/

extern int imstat(int x[]);


/*  Compare mp integers x, y, result is:	*/
/*	 4   overflow ( x or y has overflow)	*/
/*	 1   x > y				*/
/*       0   x = y				*/
/*	-1   x < y				*/

extern int imcomp(int x[], int y[], int N);


/*  Determine position of leading 1 in x, relative to mp words	*/
/*  Positions are counted from top.				*/
/*  Example for 32-bit machine word, 30 bits mp:		*/
/*								*/
/*            0   1   2   3   4   5   6   7   8			*/
/*  +---+---+---+---+---+---+---+---+---+---+---+ . . .		*/
/*  | 0 | 0 | 0 | 0 | 0 | 0 | 0 | 1 | 0 | 1 | 1 |		*/
/*  +---+---+---+---+---+---+---+---+---+---+---+ . . .		*/
/*    31  30  29  28  27  26  25  24  23  22  21		*/
/*								*/
/*  The result is 5, which is the left-shift to normalize.	*/

extern int Bitpos(int x);


/*  Normalize x > 0 so that x[1] >= (1/2)*s^Wsize [ 2^29 ]	*/
/*  Returns number of bits required for shift.  Returns -1 if	*/
/*  x <= 0.							*/

extern int imnorm(int x[], int M);
